<?php

namespace App\domain\amadeus\NDC;

use App\domain\amadeus\AmadeusRequestStateless;
use App\Traits\FileStorageTrait;
use DOMDocument;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class TravelOfferPriceAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected int $sequenceNumber;
    protected string $shoppingResponseRefID;
    protected string $offerRefID;
    protected string $shoppingOwnerCode;
    protected Collection $offerItems;

    protected string $paxListTags;
    protected array $paxOfferItemsList;

    protected string $tagNS2;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 2;

        $this->tagNS2 = 'http://xml.amadeus.com/2010/06/Travel_OfferPriceRS_v1';
    }

    public function execute(array $travelOfferRequestParam): array
    {
        try {
            $this->shoppingResponseRefID = $travelOfferRequestParam["shoppingResponseRefID"];
            $this->offerRefID = $travelOfferRequestParam["offerRefID"];
            $this->shoppingOwnerCode = $travelOfferRequestParam["shoppingOwnerCode"];
            $this->offerItems = Collection::wrap(
                isset($travelOfferRequestParam["offerItems"]["offerItemDetails"])
                    ? [$travelOfferRequestParam["offerItems"]]
                    : $travelOfferRequestParam["offerItems"]
            );

            $this->generatePaxList();

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "2-travel_offer_price_response", $response);

            $responseArray = $this->xmlResponseBodyToArrayV2($response);

            self::storeInFile(
                "json",
                "2-travel_offer_price_response",
                json_encode($responseArray)
            );


            return $responseArray;
        } catch (Exception $exception) {
            $e = "Error in TravelOfferPriceAction execution:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function xmlResponseBodyToArrayV2($xmlString)
    {
        $dom = new DOMDocument();
        $dom->loadXML($xmlString);

        $response = $dom->getElementsByTagNameNS($this->tagNS2, 'AMA_TravelOfferPriceRS')->item(0);

        // $dataList = [
        //     'BaggageAllowanceList' => $this->getElementData(
        //         $response->getElementsByTagNameNS($this->tagNS2, 'BaggageAllowanceList')->item(0)
        //     ),
        // ];

        // $dataList['BaggageAllowanceList'] = $this->extractElementsByTagName($response, "BaggageAllowance");

        $baggageAllowanceList = $response->getElementsByTagNameNS($this->tagNS2, 'BaggageAllowanceList');

        foreach ($baggageAllowanceList as $baggageAllowance) {
            $fareGroups = $baggageAllowance->getElementsByTagNameNS($this->tagNS2, 'BaggageAllowance');
            foreach ($fareGroups as $fareGroup) {
                $dataList['BaggageAllowanceList'][] = [
                    'BaggageAllowance' => $this->getElementData($fareGroup)
                ];
            }
        }

        // $fareListElements = $response->getElementsByTagNameNS($this->tagNS2, 'FareList');

        // foreach ($fareListElements as $fareListElement) {
        //     $fareGroups = $fareListElement->getElementsByTagNameNS($this->tagNS2, 'FareGroup');
        //     foreach ($fareGroups as $fareGroup) {
        //         $listKey = $fareGroup->getAttribute('ListKey'); // Get the ListKey attribute
        //         $dataList['FareList'][] = [
        //             'FareGroup' => [
        //                 'ListKey' => $listKey,
        //                 'value' => $this->getElementData($fareGroup)
        //             ]
        //         ];
        //     }
        // }

        $dataList['FareList'] = $this->extractElementsByTagName($response, "FareGroup");
        $dataList['OriginDestList'] = $this->extractElementsByTagName($response, "OriginDest");
        $dataList['PaxJourneyList'] = $this->extractElementsByTagName($response, "PaxJourney");
        $dataList['PaxList'] = $this->extractElementsByTagName($response, "Pax");
        $dataList['PaxSegmentList'] = $this->extractElementsByTagName($response, "PaxSegment");
        $dataList['PriceClassList'] = $this->extractElementsByTagName($response, "PriceClass");

        $pricedOffer['Offer'] = $this->extractElementsByTagName($response, "Offer");

        $shoppingResponse['ShoppingResponse'] = $this->extractElementsByTagName($response, "ShoppingResponseID");

        $metadata = $this->extractElementsByTagName($response, "Metadata");


        return [
            'DataLists' => $dataList,
            'Metadata' => $metadata,
            'PricedOffer' => $pricedOffer,
            'ShoppingResponse' => $shoppingResponse
        ];
    }

    protected function extractElementsByTagName($response, string $tagName): array
    {
        $elements = [];
        foreach ($response->getElementsByTagNameNS($this->tagNS2, $tagName) as $originDestElement) {
            $elements[] = $this->getElementData($originDestElement);
        }

        return $elements;
    }

    protected function getElementData($element): array
    {
        $data = array();

        // Get element attributes
        $attributes = $element->attributes;

        // Loop through attributes and add them to the data array
        foreach ($attributes as $attribute) {
            $data['@' . $attribute->name] = $attribute->value;
        }

        foreach ($element->childNodes as $child) {
            if ($child->nodeType === XML_ELEMENT_NODE) {
                // Use local name instead of full name with namespace prefix
                $childName = $child->localName;
                $childData = $this->getElementData($child);

                // If the node name already exists, ensure it is an array and add the new child data
                if (isset($data[$childName])) {
                    if (is_array($data[$childName]) && isset($data[$childName][0])) {
                        // It's already an array with multiple elements
                        $data[$childName][] = $childData;
                    } else {
                        // Convert it to an array
                        $data[$childName] = [$data[$childName], $childData];
                    }
                } else {
                    // Otherwise, just set the child data
                    $data[$childName] = $childData;
                }
            } else if ($child->nodeType === XML_TEXT_NODE) {
                // Add the text content to the array if it's not empty (avoid empty text nodes)
                $text = trim($child->nodeValue);
                if (!empty($text)) {
                    if (!isset($data['value'])) {
                        $data['value'] = $text;
                    } else {
                        if (is_array($data['value'])) {
                            $data['value'][] = $text;
                        } else {
                            $data['value'] = [$data['value'], $text];
                        }
                    }
                }
            }
        }

        return $data;
    }

    // protected function getElementData($element): array
    // {
    //     $data = array();

    //     // Get element attributes
    //     $attributes = $element->attributes;

    //     // Loop through attributes and add them to the data array
    //     foreach ($attributes as $attribute) {
    //         $data['@' . $attribute->name] = $attribute->value;
    //     }

    //     foreach ($element->childNodes as $child) {
    //         if ($child->nodeType === XML_ELEMENT_NODE) {
    //             $childData = $this->getElementData($child);

    //             // If the node name already exists, ensure it is an array and add the new child data
    //             if (isset($data[$child->nodeName])) {
    //                 if (is_array($data[$child->nodeName]) && isset($data[$child->nodeName][0])) {
    //                     // It's already an array with multiple elements
    //                     $data[$child->nodeName][] = $childData;
    //                 } else {
    //                     // Convert it to an array
    //                     $data[$child->nodeName] = [$data[$child->nodeName], $childData];
    //                 }
    //             } else {
    //                 // Otherwise, just set the child data
    //                 $data[$child->nodeName] = $childData;
    //             }
    //         } else if ($child->nodeType === XML_TEXT_NODE) {
    //             // Add the text content to the array if it's not empty (avoid empty text nodes)
    //             $text = trim($child->nodeValue);
    //             if (!empty($text)) {
    //                 if (!isset($data['value'])) {
    //                     $data['value'] = $text;
    //                 } else {
    //                     if (is_array($data['value'])) {
    //                         $data['value'][] = $text;
    //                     } else {
    //                         $data['value'] = [$data['value'], $text];
    //                     }
    //                 }
    //             }
    //         }
    //     }

    //     return $data;
    // }


    // protected function getElementData($element): array
    // {
    //     $data = array();

    //     // Get element attributes
    //     $attributes = $element->attributes;

    //     // Loop through attributes and add them to the data array
    //     foreach ($attributes as $attribute) {
    //         $data['@' . $attribute->name] = $attribute->value;
    //     }


    //     foreach ($element->childNodes as $child) {
    //         if ($child->nodeType === XML_ELEMENT_NODE) {
    //             $data[$child->nodeName] = $this->getElementData($child) ?: (string) $child;
    //         } else if ($child->nodeType === XML_TEXT_NODE) {
    //             // Add the text content to the array
    //             $data[] = $child->nodeValue;
    //         }
    //     }
    //     return $data;
    // }


    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader() . '
            <soap:Body>
               
            <AMA_TravelOfferPriceRQ xmlns="http://xml.amadeus.com/2010/06/Travel_OfferPriceRQ_v1">
                <Party>
                    <Sender>
                        <TravelAgency>
                            <AgencyID>unused</AgencyID>
                            <PseudoCityID>unused</PseudoCityID>
                        </TravelAgency>
                    </Sender>
                </Party>
                <Request>
                    <DataLists>
                        <PaxList>'
            . $this->paxListTags .
            '</PaxList>
                    </DataLists>
                    <PricedOffer>
                        <SelectedOffer>
                            <OfferRefID>' . $this->offerRefID . '</OfferRefID>
                            <OwnerCode>' . $this->shoppingOwnerCode . '</OwnerCode>
                            <ShoppingResponseRefID>' . $this->shoppingResponseRefID . '</ShoppingResponseRefID>

                            ' . $this->generateOfferItemTags() . '

                        </SelectedOffer>
                    </PricedOffer>
                </Request>
            </AMA_TravelOfferPriceRQ>

            </soap:Body>

        </soap:Envelope>';

        self::storeInFile("xml", "2-travel_offer_price_request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/Travel_OfferPrice_1.2";
    }

    protected function generatePaxList(): void
    {
        $xml = "";

        $adults = $this->offerItems->where("paxReference.ptc", "ADT")->first();
        $childs = $this->offerItems->where("paxReference.ptc", "CHD")->first();
        $infants = $this->offerItems->where("paxReference.ptc", "INF")->first();

        $ptsCount = 0;

        $adtString = "";
        $infString = "";

        $paxOfferItem = [];

        if ($adults) {
            $travellers = isset($adults["paxReference"]["traveller"]["ref"])
                ? [$adults["paxReference"]["traveller"]]
                : $adults["paxReference"]["traveller"];

            $ptc = $adults["paxReference"]["ptc"];

            $infTravellerXMLArray = [];
            if ($infants) {
                $infTravellers = isset($infants["paxReference"]["traveller"]["ref"])
                    ? [$infants["paxReference"]["traveller"]]
                    : $infants["paxReference"]["traveller"];

                foreach ($infTravellers as $infTraveller) {
                    $infTravellerXMLArray[] = "<PTC>INF</PTC>";
                }
            }

            $infCount = 0;
            $ptcRefIds = [];
            $infRefIds = [];

            foreach ($travellers as $index => $traveller) {
                $adtString .= "
                <Pax>
                ";

                $ptsCount++;
                $ptcRefId = "P$ptsCount";

                $adtString .= "
                            <PaxID>$ptcRefId</PaxID>
                            <PTC>$ptc</PTC>
                        ";

                if (isset($infTravellerXMLArray[$index])) {
                    $infCount++;
                    $infRef = "P$ptsCount.$infCount";
                    $infRefIds[] = $infRef;

                    $adtString .= "<PaxRefID>$infRef</PaxRefID>";

                    $infString .= "
                        <Pax>
                            <PaxID>$infRef</PaxID>
                            <PTC>INF</PTC>
                        </Pax>
                    ";
                }

                $adtString .= "
                </Pax>
                ";

                $ptcRefIds[] = $ptcRefId;
            }

            $paxOfferItem[] = [
                "OfferItemRefID" => $adults["offerItemDetails"]["offerItemId"],
                "PaxRefID" => implode(' ', $ptcRefIds)
            ];

            if ($infants) {
                $paxOfferItem[] = [
                    "OfferItemRefID" => $infants["offerItemDetails"]["offerItemId"],
                    "PaxRefID" => implode(' ', $infRefIds)
                ];
            }
        }

        $chdString = "";
        if ($childs) {
            $travellers = isset($childs["paxReference"]["traveller"]["ref"])
                ? [$childs["paxReference"]["traveller"]]
                : $childs["paxReference"]["traveller"];

            $ptc = $childs["paxReference"]["ptc"];

            $chdRefIds = [];

            foreach ($travellers as $traveller) {
                $ptsCount++;
                $ptcRefId = "P$ptsCount";
                $chdRefIds[] = $ptcRefId;

                $chdString .= "
                        <Pax>
                            <PaxID>$ptcRefId</PaxID>
                            <PTC>$ptc</PTC>
                        </Pax>
                        ";
            }

            $paxOfferItem[] = [
                "OfferItemRefID" => $childs["offerItemDetails"]["offerItemId"],
                "PaxRefID" => implode(' ', $chdRefIds)
            ];
        }


        $xml .= $adtString;
        $xml .= $infString;
        $xml .= $chdString;


        $this->paxListTags = $xml;
        $this->paxOfferItemsList = $paxOfferItem;
    }

    protected function generateOfferItemTags(): string
    {
        $xml = "";

        foreach ($this->paxOfferItemsList as $offerItem) {
            $xml .= "
                <SelectedOfferItem>
                    <OfferItemRefID>" . $offerItem["OfferItemRefID"]  . "</OfferItemRefID>
                    <OwnerCode>" . $this->shoppingOwnerCode . "</OwnerCode>
                    <PaxRefID>" . $offerItem["PaxRefID"] . "</PaxRefID>

                </SelectedOfferItem>
            ";
        }

        return $xml;
    }
}
