<?php

namespace App\domain\amadeus\NDC;

use App\domain\amadeus\AmadeusRequestStateless;
use App\Traits\FileStorageTrait;
use DOMDocument;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class TravelOrderCreateAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected int $sequenceNumber;
    protected string $pricingOfferId;
    protected string $pricingOwnerCode;
    protected string $pricingResponseId;
    protected array $offerItems;
    protected Collection $travellers;
    protected array $paxList;

    protected string $paxListTags;
    protected array $paxOfferItemsList;

    protected string $tagNS2;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 2;

        $this->tagNS2 = 'http://xml.amadeus.com/2010/06/Travel_OrderViewRS_v1';
    }

    public function execute(array $travelOrderParams): array
    {
        try {
            $this->pricingOfferId = $travelOrderParams["PricingOfferID"];
            $this->pricingOwnerCode = $travelOrderParams["PricingOwnerCode"];
            $this->pricingResponseId = $travelOrderParams["PricingResponseID"];
            $this->offerItems = $travelOrderParams["OfferItem"];
            $this->paxList = $travelOrderParams["PaxList"];
            $this->travellers = Collection::wrap($travelOrderParams["Travellers"]);


            $response = $this->createCurlRequest();

            self::storeInFile("xml", "3-travel_order_create_response", $response);

            $responseArray = $this->xmlResponseBodyToArrayV2($response);

            self::storeInFile(
                "json",
                "3-travel_order_create_response",
                json_encode($responseArray)
            );

            return $responseArray;
        } catch (Exception $exception) {
            $e = "Error in Travel Order Create Action execution:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function xmlResponseBodyToArrayV2($xmlString)
    {
        $dom = new DOMDocument();
        $dom->loadXML($xmlString);

        $response = $dom->getElementsByTagNameNS($this->tagNS2, 'AMA_TravelOrderViewRS')->item(0);

        $error = $this->extractElementsByTagName($response, "Errors");

        if(count($error) > 0){
            return ["error"=> $error];
        }

        $dataList = $this->extractElementsByTagName($response, "Response");

        return [
            'Response' => $dataList,
        ];
    }

    protected function extractElementsByTagName($response, string $tagName): array
    {
        $elements = [];
        foreach ($response->getElementsByTagNameNS($this->tagNS2, $tagName) as $originDestElement) {
            $elements[] = $this->getElementData($originDestElement);
        }

        return $elements;
    }

    protected function getElementData($element): array
    {
        $data = array();

        // Get element attributes
        $attributes = $element->attributes;

        // Loop through attributes and add them to the data array
        foreach ($attributes as $attribute) {
            $data['@' . $attribute->name] = $attribute->value;
        }

        foreach ($element->childNodes as $child) {
            if ($child->nodeType === XML_ELEMENT_NODE) {
                // Use local name instead of full name with namespace prefix
                $childName = $child->localName;
                $childData = $this->getElementData($child);

                // If the node name already exists, ensure it is an array and add the new child data
                if (isset($data[$childName])) {
                    if (is_array($data[$childName]) && isset($data[$childName][0])) {
                        // It's already an array with multiple elements
                        $data[$childName][] = $childData;
                    } else {
                        // Convert it to an array
                        $data[$childName] = [$data[$childName], $childData];
                    }
                } else {
                    // Otherwise, just set the child data
                    $data[$childName] = $childData;
                }
            } else if ($child->nodeType === XML_TEXT_NODE) {
                // Add the text content to the array if it's not empty (avoid empty text nodes)
                $text = trim($child->nodeValue);
                if (!empty($text)) {
                    if (!isset($data['value'])) {
                        $data['value'] = $text;
                    } else {
                        if (is_array($data['value'])) {
                            $data['value'][] = $text;
                        } else {
                            $data['value'] = [$data['value'], $text];
                        }
                    }
                }
            }
        }

        return $data;
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader() . '
            <soap:Body>
               
            <AMA_TravelOrderCreateRQ xmlns="http://xml.amadeus.com/2010/06/Travel_OrderCreateRQ_v1">
                <Party>
                <Sender>
                    <TravelAgency>
                    <AgencyID>unused</AgencyID>
                    <PseudoCityID>unused</PseudoCityID>
                    </TravelAgency>
                </Sender>
                </Party>
                <Request>
                <CreateOrder>
                    <SelectedOffer>
                    <OfferID>' . $this->pricingOfferId . '</OfferID>
                    <OwnerCode>' . $this->pricingOwnerCode . '</OwnerCode>
                    <ShoppingResponseRefID>' . $this->pricingResponseId . '</ShoppingResponseRefID>
                    
                    ' . $this->generatePaxWiseOfferItems() . '

                    </SelectedOffer>
                </CreateOrder>
                <DataLists>
                    <PaxList>

                    ' . $this->generatedPaxList() . '

                    </PaxList>
                </DataLists>
                </Request>
            </AMA_TravelOrderCreateRQ>

            </soap:Body>

        </soap:Envelope>';

        self::storeInFile("xml", "3-travel_order_create_request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/Travel_OrderCreate_1.6";
    }

    protected function generatePaxWiseOfferItems()
    {
        $xml = "";

        $paxIds = [];

        $paxIds[] = $this->filterPax("ADT")->pluck('PaxID.value');
        if ($this->filterPax("CHD")->count() > 0) {
            $paxIds[] = $this->filterPax("CHD")->pluck('PaxID.value');
        }
        if ($this->filterPax("INF")->count() > 0) {
            $paxIds[] = $this->filterPax("INF")->pluck('PaxID.value');
        }

        foreach ($paxIds as $paxId) {
            $offerItemId = $this->getOfferItemId($paxId);

            $xml .= "
                    <SelectedOfferItem>
                        <OfferItemID>" . $offerItemId . "</OfferItemID>
            ";

            foreach ($paxId as $id) {
                $xml .= "<PaxRefID>" . $id . "</PaxRefID>";
            }

            $xml .= " </SelectedOfferItem>";
        }

        return $xml;
    }

    protected function getOfferItemId($paxIds)
    {
        $offerItemId = "";

        if (isset($this->offerItems["OfferItemID"])) {
            $offerItemId = $this->offerItems["OfferItemID"]["value"];
        } else {
            $relevantOfferItem = null;

            foreach ($this->offerItems as $offerItem) {
                $passengerRefs = data_get($offerItem, 'FareDetail.PassengerRefs');

                if (isset($passengerRefs["value"])) {
                    if (data_get($passengerRefs, 'value') === $paxIds[0]) {
                        $relevantOfferItem = $offerItem;

                        break;
                    }
                } else {
                    foreach ($passengerRefs as $ref) {
                        if (data_get($ref, 'value') === $paxIds[0]) {
                            $relevantOfferItem =  $offerItem;

                            break 2;
                        }
                    }
                }
            }

            $offerItemId = $relevantOfferItem ? $relevantOfferItem["OfferItemID"]["value"] : "";
        }

        return $offerItemId;
    }

    protected function filterByPassengerRef($collection, $passengerRef)
    {
        return $collection->filter(function ($item) use ($passengerRef) {
            Log::info('$item', $item);
            $passengerRefs = data_get($item, 'FareDetail.PassengerRefs');

            Log::info('$passengerRefs', $passengerRefs);
            if (is_array($passengerRefs)) {
                foreach ($passengerRefs as $ref) {
                    Log::info('-->data_get--> ' . data_get($ref, 'value'));
                    Log::info('-->$passengerRef--> ' . $passengerRef);
                    if (data_get($ref, 'value') === $passengerRef) {
                        return true;
                    }
                }
            } else {
                Log::info('-->data_get --> ' . $passengerRefs);
                Log::info('-->$passengerRef--> ' . $passengerRef);
                return data_get($passengerRefs, 'value') === $passengerRef;
            }
            return false;
        });
    }

    protected function generatedPaxList()
    {
        $xml = "";

        $adultTraveller = $this->filterTraveller("ADT");
        $childTraveller = $this->filterTraveller("CHD");
        $infantTraveller = $this->filterTraveller("INF");

        $adultPax = $this->filterPax("ADT");
        $childPax = $this->filterPax("CHD");
        $infantPax = $this->filterPax("INF");

        foreach ($adultTraveller as $index => $traveller) {

            $trav =  $traveller["Traveler"];

            $xml .= "
                    <Pax>
                        <PaxID>" . $adultPax[$index]["PaxID"]["value"] . "</PaxID>
                        <PTC>" . $adultPax[$index]["PTC"]["value"] . "</PTC>
                        <Birthdate>" . $trav["birthDate"]  . "</Birthdate>

                        " . $this->generatePaxRefIdForInfant($infantPax, $index) . "

                        <ContactInfo>
                            <ContactRefusedInd>false</ContactRefusedInd>
                            <Phone>
                                <LabelText>MOBILE</LabelText>
                                <PhoneNumber>" . $trav["Telephone"][0]["phoneNumber"]  . "</PhoneNumber>
                            </Phone>
                        </ContactInfo>
                        <ContactInfo>
                            <ContactRefusedInd>false</ContactRefusedInd>
                            <EmailAddress>
                                <LabelText>EMAIL</LabelText>
                                <EmailAddressText>" . $trav["Email"][0]["value"] . "</EmailAddressText>
                            </EmailAddress>
                        </ContactInfo>
                        
                        " . $this->generateAddressTags($trav) . "

                        <IdentityDoc>
                            <IdentityDocID>" . $trav["TravelDocument"][0]["docNumber"] . "</IdentityDocID>
                            <IdentityDocTypeCode>" . ($trav["TravelDocument"][0]["docType"] === "Passport" ? "PT" : "P") . "</IdentityDocTypeCode>
                            <IssuingCountryCode>" . $trav["TravelDocument"][0]["issueCountry"] . "</IssuingCountryCode>
                            <ExpiryDate>" . $trav["TravelDocument"][0]["expireDate"] . "</ExpiryDate>                 
                        </IdentityDoc>
                        <Individual>
                            <GenderCode>" . ($trav["gender"] === "Male" ? "M" : "F") . "</GenderCode>
                            <TitleName>" . $trav["PersonName"]["Title"] . "</TitleName>
                            <GivenName>" . $trav["PersonName"]["Given"] . "</GivenName>
                            <Surname>" . $trav["PersonName"]["Surname"] . "</Surname>
                        </Individual>              
                    </Pax>

                    " .  $this->generateInfantPaxTag($infantTraveller, $infantPax, $index) . "

            ";
        }

        $xml .= $this->generateChildPaxTag($childTraveller, $childPax);

        return $xml;
    }

    protected function generatePaxRefIdForInfant(Collection $infantPax, int $index): string
    {
        $xml = "";
        if ($infantPax->has($index)) {
            $xml .=  "<PaxRefID>" . $infantPax[$index]["PaxID"]["value"] . "</PaxRefID>";
        }

        return $xml;
    }

    protected function generateAddressTags(array $traveller)
    {
        $xml = "";

        $postalAddress = isset($traveller["PostalAddress"]) ? $traveller["PostalAddress"] : null;
        $deliveryAddress = isset($traveller["DeliveryAddress"]) ? $traveller["DeliveryAddress"] : null;

        $addresses = [];

        if ($postalAddress !== null) {
            $addresses[] = $postalAddress;
        }

        if ($deliveryAddress !== null) {
            $addresses[] = $deliveryAddress;
        }

        foreach ($addresses as $address) {
            $xml .= "
                <ContactInfo>
                    <ContactRefusedInd>false</ContactRefusedInd>
                    <PostalAddress>
                        <LabelText>" . $address["LabelText"] . "</LabelText>
                        <StreetText>" . $address["StreetText"] . "</StreetText>
                        <PostalCode>" . $address["PostalCode"] . "</PostalCode>
                        <CityName>" . $address["CityName"] . "</CityName>
                        <CountrySubDivisionName>" . $address["CountrySubDivisionName"] . "</CountrySubDivisionName>
                        <CountryCode>" . $address["CountryCode"] . "</CountryCode>
                    </PostalAddress>
                </ContactInfo> 
            ";
        }

        return $xml;
    }

    protected function filterTraveller(string $ptc): Collection
    {
        return $this->travellers->filter(function ($item) use ($ptc) {
            return $item['Traveler']['passengerTypeCode'] ===  $ptc;
        })
            ->values();
    }

    protected function filterPax(string $ptc): Collection
    {
        return Collection::wrap($this->paxList)->filter(function ($item) use ($ptc) {
            return $item['PTC']['value'] ===  $ptc;
        })
            ->values();
    }

    protected function generateInfantPaxTag(Collection $infantDetails, Collection $paxIds, int $indexNo)
    {
        if ($infantDetails->has($indexNo) && $paxIds->has($indexNo)) {
            $trav =  $infantDetails[$indexNo]["Traveler"];
            $pax = $paxIds[$indexNo];

            return $this->generatePaxTag($trav, $pax);
        } else {
            return "";
        }
    }

    protected function generateChildPaxTag(Collection $childrensDetails, Collection $paxIds)
    {
        $xml = "";

        foreach ($childrensDetails as $index => $child) {
            $trav =  $child["Traveler"];
            $pax = $paxIds[$index];

            $xml .= $this->generatePaxTag($trav, $pax);
        }

        return $xml;
    }

    protected function generatePaxTag($traveller, $pax)
    {
        return "
            <Pax>
                <PaxID>" . $pax["PaxID"]["value"]  . "</PaxID>
                <PTC>" . $pax["PTC"]["value"] . "</PTC>
                <Birthdate>" . $traveller["birthDate"]  . "</Birthdate>
                <IdentityDoc>
                    <IdentityDocID>" . $traveller["TravelDocument"][0]["docNumber"] . "</IdentityDocID>
                    <IdentityDocTypeCode>" . ($traveller["TravelDocument"][0]["docType"] === "Passport" ? "PT" : "P") . "</IdentityDocTypeCode>
                    <IssuingCountryCode>" . $traveller["TravelDocument"][0]["issueCountry"] . "</IssuingCountryCode>
                    <ExpiryDate>" . $traveller["TravelDocument"][0]["expireDate"] . "</ExpiryDate>
                </IdentityDoc>
                <Individual>
                    <GenderCode>" . ($traveller["gender"] === "Male" ? "M" : "F") . "</GenderCode>
                    <GivenName>" . $traveller["PersonName"]["Given"] . "</GivenName>
                    <Surname>" . $traveller["PersonName"]["Surname"] . "</Surname>
                </Individual>  
            </Pax>
        ";
    }
}
