<?php

namespace App\domain\amadeus\NDC;

use App\domain\amadeus\AmadeusRequestStateless;
use App\Traits\FileStorageTrait;
use DOMDocument;
use Exception;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Log;

class TravelOrderPayAction extends AmadeusRequestStateless
{
    use FileStorageTrait;

    protected int $sequenceNumber;
    protected string $orderId;
    protected string $ownerCode;
    protected string $currencyCode;
    protected float $totalAmount;
    protected string $tagNS2;

    public function __construct()
    {
        parent::__construct();

        $this->sequenceNumber = 2;

        $this->tagNS2 = 'http://xml.amadeus.com/2010/06/Travel_OrderViewRS_v1';
    }

    public function execute(array $travelOrderParams): array
    {
        try {
            $this->orderId = $travelOrderParams["OrderID"];
            $this->ownerCode = $travelOrderParams["OwnerCode"];
            $this->totalAmount = $travelOrderParams["TotalAmount"];
            $this->currencyCode = $travelOrderParams["CurrencyCode"];

            $response = $this->createCurlRequest();

            self::storeInFile("xml", "4-travel_order_pay_response", $response);

            $responseArray = $this->xmlResponseBodyToArrayV2($response);

            self::storeInFile("json", "4-travel_order_pay_response", json_encode($responseArray));

            return $responseArray;
        } catch (Exception $exception) {
            $e = "Error in Travel Order Pay Action execution:" . $exception->getMessage();
            Log::error($e);
            echo $e;

            return ["error" => $e];
        }
    }

    protected function xmlResponseBodyToArrayV2($xmlString)
    {
        $dom = new DOMDocument();
        $dom->loadXML($xmlString);

        $response = $dom->getElementsByTagNameNS($this->tagNS2, 'AMA_TravelOrderViewRS')->item(0);


        $dataList = $this->extractElementsByTagName($response, "Response");

        $error = $this->extractElementsByTagName($response, "Errors");

        if(count($error) > 0){
            return ["error"=> $error];
        }

        return [
            'Response' => $dataList,
        ];
    }

    protected function extractElementsByTagName($response, string $tagName): array
    {
        $elements = [];
        foreach ($response->getElementsByTagNameNS($this->tagNS2, $tagName) as $originDestElement) {
            $elements[] = $this->getElementData($originDestElement);
        }

        return $elements;
    }

    protected function getElementData($element): array
    {
        $data = array();

        // Get element attributes
        $attributes = $element->attributes;

        // Loop through attributes and add them to the data array
        foreach ($attributes as $attribute) {
            $data['@' . $attribute->name] = $attribute->value;
        }

        foreach ($element->childNodes as $child) {
            if ($child->nodeType === XML_ELEMENT_NODE) {
                // Use local name instead of full name with namespace prefix
                $childName = $child->localName;
                $childData = $this->getElementData($child);

                // If the node name already exists, ensure it is an array and add the new child data
                if (isset($data[$childName])) {
                    if (is_array($data[$childName]) && isset($data[$childName][0])) {
                        // It's already an array with multiple elements
                        $data[$childName][] = $childData;
                    } else {
                        // Convert it to an array
                        $data[$childName] = [$data[$childName], $childData];
                    }
                } else {
                    // Otherwise, just set the child data
                    $data[$childName] = $childData;
                }
            } else if ($child->nodeType === XML_TEXT_NODE) {
                // Add the text content to the array if it's not empty (avoid empty text nodes)
                $text = trim($child->nodeValue);
                if (!empty($text)) {
                    if (!isset($data['value'])) {
                        $data['value'] = $text;
                    } else {
                        if (is_array($data['value'])) {
                            $data['value'][] = $text;
                        } else {
                            $data['value'] = [$data['value'], $text];
                        }
                    }
                }
            }
        }

        return $data;
    }

    protected function generateXMLRequest(): string
    {
        $xmlRequest = $this->xmlHeader() . '
            <soap:Body>
                <AMA_TravelOrderPayRQ xmlns="http://xml.amadeus.com/2010/06/Travel_OrderPayRQ_v1">
                    <Party>
                    <Sender>
                        <TravelAgency>
                        <AgencyID>unused</AgencyID>
                        <PseudoCityID>unused</PseudoCityID>
                        </TravelAgency>
                    </Sender>
                    </Party>
                    <Request>
                    <PaymentInfo>
                        <Amount CurCode="' . $this->currencyCode . '">' . $this->totalAmount . '</Amount>
                        <TypeCode>CA</TypeCode>
                        <PaymentMethod>
                        <Cash>
                            <CashInd>true</CashInd>
                        </Cash>
                        </PaymentMethod>
                    </PaymentInfo>
                    <Order>
                        <OrderID>' . $this->orderId . '</OrderID>
                        <OwnerCode>' . $this->ownerCode . '</OwnerCode>
                    </Order>
                    </Request>
                </AMA_TravelOrderPayRQ>
            </soap:Body>

        </soap:Envelope>';

        self::storeInFile("xml", "4-travel_order_pay_request", $xmlRequest);

        return $xmlRequest;
    }

    protected function action(): string
    {
        return "http://webservices.amadeus.com/Travel_OrderPay_1.6";
    }
}
